//---------------------------------------------------------------------------
// Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
//  I2CMaster.c - Library file for I2C master routines.
//
//  Date: June 21, 2004
//  Version: 1.00
//
// --------------------------------------------------------------------------

#include "I2Cmaster.h"

dbyte I2CBus_txBuffer;
bit I2CBus_rxAck;
dbyte I2CBus_rxBuffer;
bit I2CBus_txAck;

static dbyte I2CBus_clockDelay;
static dbyte I2CBus_slaveAddress;
static dbyte I2CBus_cnt;
static dbyte I2CBus_mask;

DECLARE_BIT(P0,2,SDA);
DECLARE_BIT(P0,3,SCL);

#define WAIT_CLOCK() {usDelay(I2CBus_clockDelay);}
#define WAIT_SHORT() {usDelay(1);}
#define WAIT_LONG()  {usDelay((I2CBus_clockDelay<<1));}
#define SCL_HIGH()   {SCL=1; while( !SCL ) /*no-op*/;}
#define SCL_LOW()    {SCL=0; while(  SCL ) /*no-op*/;}
#define SDA_HIGH()   {SDA=1; while( !SDA ) /*no-op*/;}
#define SDA_LOW()    {SDA=0; while(  SDA ) /*no-op*/;}
/*
void printHex(uchar* buff, uchar len)
{
   uchar i;
   for(i=0; i<len; i++)
      printf("%02X ", (int)buff[i]);
   printf("\n");
}

void printHexLabeled(char* label, uchar* buff, uchar len)
{
   printf("%s: ", label);
   printHex(buff, len);
}
*/
void I2CBus_init()
{
   I2CBus_clockDelay = 10;
   SCL_HIGH();
   while(SDA==0)
   {
      SCL_HIGH();
      SCL_LOW();
   }
}

void I2CBus_setClockDelay(uchar delay)
{
   I2CBus_clockDelay = delay;
}

void I2CBus_setSlaveAddress(uchar addr)
{
   I2CBus_slaveAddress = addr;
}

void I2CBus_start()
{
   //printf("   start\n");
   SCL_HIGH();
   SDA_HIGH();
   WAIT_CLOCK();
   SDA_LOW();
   WAIT_CLOCK();
   SCL_LOW();
   WAIT_LONG();
}

void I2CBus_stop()
{
   SCL_LOW();
   SDA_LOW();
   WAIT_CLOCK();
   SCL_HIGH();
   WAIT_CLOCK();
   SDA_HIGH();
   WAIT_LONG();
   //printf("   stop\n");
}

void I2CBus_putByte()
{
   //printf("      put: %02X, ", (int)I2CBus_txBuffer);
   SCL_LOW();

   // transmit the byte
   for(I2CBus_mask = 0x80; I2CBus_mask>0; I2CBus_mask>>=1)
   {
      WAIT_CLOCK();
      if((I2CBus_txBuffer&I2CBus_mask)>0)
      {
         SDA_HIGH();
      }
      else
      {
         SDA_LOW();
      }
      SCL_HIGH();
      WAIT_CLOCK();
      SCL_LOW();
   }

   // wait for the ack
   SDA = 1;
   WAIT_CLOCK();
   SCL_HIGH();
   I2CBus_rxAck = !SDA;
   WAIT_CLOCK();
   SCL_LOW();

   //printf((I2CBus_rxAck)?"ACK\n":"NACK\n");

   WAIT_LONG();
}

void I2CBus_getByte()
{
   SDA = 1;
   SCL_LOW();
   WAIT_CLOCK();
   I2CBus_rxBuffer = 0;
   for(I2CBus_mask = 0x80; I2CBus_mask>0; I2CBus_mask>>=1)
   {
      SCL_HIGH();
      WAIT_CLOCK();
      if(SDA)
         I2CBus_rxBuffer |= I2CBus_mask;
      SCL_LOW();
      WAIT_CLOCK();
   }

   if(I2CBus_txAck)
   {
      SDA_LOW();
      SCL_HIGH();
      WAIT_CLOCK();
      SCL_LOW();
      WAIT_CLOCK();
      SDA = 1; //SDA_HIGH();
   }
   //printf("      get: %02X\n", (int)I2CBus_rxBuffer);
}

bit I2CBus_write(uchar* txBytes, uchar len)
{
   //printHexLabeled("writing", txBytes, len);
   I2CBus_start();
   I2CBus_txBuffer = I2CBus_slaveAddress & 0xFE;
   I2CBus_putByte();
   if(I2CBus_rxAck)
   {
      for(I2CBus_cnt=0; I2CBus_cnt<len; I2CBus_cnt++)
      {
         I2CBus_txBuffer = txBytes[I2CBus_cnt];
         I2CBus_putByte();
         if(!I2CBus_rxAck)
            break;
      }
   }
   I2CBus_stop();
   return I2CBus_rxAck;
}

bit I2CBus_read(uchar* rxBytes, int len)
{
   I2CBus_start();
   I2CBus_txBuffer = I2CBus_slaveAddress | 0x01;
   I2CBus_putByte();
   if(I2CBus_rxAck && len>0)
   {
      // transmit Ack for all but the last byte
      I2CBus_txAck = 1;
      len = len - 1;
      for(I2CBus_cnt=0; I2CBus_cnt<len; I2CBus_cnt++)
      {
         I2CBus_getByte();
         rxBytes[I2CBus_cnt] = I2CBus_rxBuffer;
      }
      // Nack for the last byte
      I2CBus_txAck = 0;
      I2CBus_getByte();
      rxBytes[I2CBus_cnt] = I2CBus_rxBuffer;
   }
   I2CBus_stop();
   //printHexLabeled("reading", rxBytes, len+1);
   return I2CBus_rxAck;
}


