//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//--------------------------------------------------------------------------

// file ecash.h
#include "reg420.h"
#include "ecerror.h"
#include "ecow.h"
#include <string.h>
#include <ctype.h>

#define FirmwareString "eCash_Kit  Rev A"

//#define USE_DS2460

/*
Explanation of different memory locations recognized by C51:
   code - program memory; accessed by opcode MOVC @A+DPTR (Up to 64 KBytes)
   data - directly addressable internal data memory; fastest access to variables (128 bytes)
   idata - indirectly addressable internal data memory; accessed across the full internal address space (256 bytes)
   bdata - bit-addressable internal data memory; allows mixed bit and byte access (16 bytes)
   xdata - external data memory; accessed by opcode MOVX @DPTR (Up to 64 KBytes)
   pdata - paged external data memory; accessed by opcode MOVX @Rn (256 bytes)

Explanation of different data types and their size for C51:
   char - 1 byte (signed/unsigned)
   enum - 2 bytes
   short - 2 bytes (signed/unsigned)
   int - 2 bytes (signed/unsigned)
   long - 4 bytes (signed/unsigned)
   float - 4 bytes (1.175494E-38 to 3.402823E+38)
   bit - 1 bit
   sbit - 1 bit
   sfr - 1 byte
   sfr16 - 2 bytes
*/

// COMMONLY USED DATA TYPES
#define uchar unsigned char
#define cbyte code unsigned char
// use to declare a bit-addressable byte
#define bbyte bdata unsigned char
// use to declare a scratchpad byte
#define dbyte data unsigned char
// use to declare a scratchpad integer
#define dint data unsigned int
// use to declare an XData byte
#define xbyte xdata unsigned char
// use to declare a bit
#define DECLARE_BIT(byteAddress,bitNumber,bitLabel) sbit bitLabel = byteAddress^bitNumber

// Number of times to retry 1-Wire operations.  0 will do max (256)
#define RetryCount 0

#define DODEBUG 0

#if DODEBUG
   #define DEBUG(str1) serial_putstring(str1, sizeof(str1)-1)
   #define DEBUGLN(str1) DEBUG(str1"\n")
   #define DEBUGBLOCK(block, size) serial_puthexstring(block, size)
   #define DEBUGBLOCKLBL(str1, block, size) {DEBUG(str1); DEBUGBLOCK(block, size); DEBUG("\n");}
#else
   #define DEBUG(str1) //nothing
   #define DEBUGLN(str1) //nothing
   #define DEBUGBLOCK(block, size) //nothing
   #define DEBUGBLOCKLBL(str1, block, size) //nothing
#endif

#define PRINT(str1) serial_putstring(str1, sizeof(str1)-1)
#define PRINTLN(str1) PRINT(str1"\n")
#define PRINTBLOCK(block, size) serial_puthexstring(block, size)
#define PRINTBLOCKLBL(str1, block, size) {PRINT(str1); PRINTBLOCK(block, size); PRINT("\n");}

#define LCD_PRINT(str1) lcd_putstring(str1, sizeof(str1)-1);

#define WAIT_SECS(secs) secTimerRun=0; wait2ms(); TmrSeconds=secs; secTimerRun=1; TimeOut=0; while(!TimeOut);

#define TIMEOUT_VALUE_SECS 15

#define DS5000 5000
#define DS89C420 420
#define PROCESSOR DS89C420


// One Wire Commands
#define readByteCommand 0x96
#define writeByteCommand 0xA5
#define resetOneWireCommand 0xB4
#define onewireBitCommand 0x87


// DS2482 Commands and pointers
#define setReadPointerCommand 0xE1
#define readDataRegister 0xE1
#define writeConfigCommand 0xD2
#define statusRegister 0xF0
#define configRegister 0xC3
#define deviceReset 0xF0
 
// 1-Wire levels
#define NORMAL 0
#define POWER_DELIV 1

// 1-Wire speeds
#define Overdrive 1
#define Standard 0

// Boolean varialbles
#define FAILURE 0
#define SUCCESS 1

#ifndef OSCILLATOR
//   #define OSCILLATOR 33000000
//   #define OSCILLATOR 22118400
//   #define OSCILLATOR 14745600
//   #define OSCILLATOR 14318000
   #define OSCILLATOR 11059200
#endif

// Crystal multiplier for the 420.  By default, the 420 is in 1x mode,
// but it supports both 2x and 4x multipliers.  BUT, the timers are
// still looking at the external clock frequency, so we need to know
// if we are multiplying
//#define CTM2
#ifdef CTM2
   #define CTMDIV 2.0
#else
   #ifdef CTM4
      #define CTMDIV 4.0
   #else
      #define CTMDIV 1.0
   #endif
#endif


// Assumes SMOD=1
#define T1DIV 12.0
#define BAUD9600_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*9600)))
#define BAUD19200_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*19200)))
#define BAUD57600_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*57600)))
#define BAUD115200_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*115200)))

#define SInBufSize 128
#define SOutBufSize 128

// Timer 0 reload value for 1kHz ticks
// reload value = 65536 - (OSC/12)/1kHz
// assumes T0DIV=12
#define TIMER0_1MSEC_RELOAD ((int)(65536-((OSCILLATOR/CTMDIV/12.0)/1000L)))

/*
// file ecsearch.c
extern void owNext(void);
extern void owResetSearch(void);

// file ecserial.c
extern void serial_putstring(uchar*, uchar);
extern void serial_puthexstring(uchar*, uchar);
extern void serial_init(void);
extern char putchar(char);
*/
// file ectimer.c
extern void timer0Init(void);
extern void startTimerSec(void);
extern void stopTimerSec(void);
extern void startTimerMS(void);
extern void stopTimerMS(void);
extern void wait2ms(void);
extern void wait10ms(void);
extern void wait250ms(void);
extern void halfUsDelay();
extern void usDelay(unsigned int);
extern void msDelay(unsigned int);

extern dbyte milliseconds; // counts down millseconds
extern dbyte QtrSeconds; // counts down each quarter second
extern bit SecBit; // Toggles every Hz
extern bit secTimerRun; // Set when TmrSeconds is counting down
extern dbyte TmrSeconds; // Counts down at 1Hz
extern bit TimeOut; // Set when Seconds value has unflowed
extern bit msTimerRun; // Set when msCounter is counting up
extern dbyte msCounter; // Counts up at 1kHz
extern bit msTimerOverflow; // Set when timer runs for more then 255 ms

// file eclcd.c
extern void lcd_init(void);
extern void lcd_putstring(uchar*, uchar);
extern void lcd_putchar(char);
extern void lcd_clear();
extern void lcd_home();

extern xbyte SearchSerialNum[8]; // Rom ID generated from search routine   

